<?php
/*
* @version      2.0.4 06.09.2023
* @author       MAXXmarketing GmbH
* @package      addon_api
* @copyright    Copyright (C) 2010 webdesigner-profi.de. All rights reserved.
* @license      GNU/GPL
*/
namespace Joomla\Component\Jshopping\Site\Model\Api;
defined('_JEXEC') or die;

class CategoryModel extends ApiModel {

    public function ids(): array {
        $db = \JFactory::getDbo();
        $db->setQuery('
            SELECT ' . $db->qn('category_id') . '
            FROM '   . $db->qn('#__jshopping_categories')
        );
        $res = array_map('intval', (array) $db->loadColumn());
        sort($res);
        return $res;
    }

    /**
     * @throws AddonApiException
     */
    public function item(int $id, int $extra_data = 0): array {
        $user        = \JFactory::getUser();
        $jshopConfig = \JSFactory::getConfig();
        $dispatcher  = \JFactory::getApplication();
        $category    = \JSFactory::getTable('category');
        $category->load($id);
        if (!$category->category_id) {
            throw new \AddonApiException_category(1, ' ' . $id);
        }
        $category->getDescription();
        $dispatcher->triggerEvent('onAfterLoadCategory', [&$category, &$user]);
        $sub_categories = $category->getChildCategories(
            $category->getFieldListOrdering(),
            $category->getSortingDirection(),
            1
        );
        $dispatcher->triggerEvent('onBeforeDisplayCategory', [&$category, &$sub_categories]);
        if ($extra_data) {
            $res  = [
                'category'                 =>          $category,
                'count_category_to_row'    => (int)    $category->getCountToRow(),
                'count_product_to_row'     => (int)    $category->products_row,
                'filter_show'              =>          true,
                'filter_show_category'     =>          false,
                'filter_show_manufacturer' =>          true,
                'image_category_path'      => (string) $jshopConfig->image_category_live_path,
                'manufacuturers'           => (array)  $category->getManufacturers(),
                'noimage'                  => (string) $jshopConfig->noimage,
                'subcategories'            => (array)  $sub_categories,
            ];
        } else {
            $res  = [
                'category'                 =>          $category,              
            ];
        }
        return $res;
    }

    public function tree(): array {
        $parents = [];
        foreach (\JSFactory::getTable('category')->getAllCategories(1,1,'name') as $category) {
            $parent_id             = (int) $category->category_parent_id;
            $parents[$parent_id][] = [
                'id'            => (int) $category->category_id,
                'parent_id'     => $parent_id,                
                'name' => $category->name,
                'subcategories' => [],
            ];
        }
        ksort($parents);
        $create_node = function($items) use(&$create_node, $parents) {
            $node = [];
            foreach ($items as $item) {
                if (isset($parents[$item['id']])) {
                    $item['subcategories'] = $create_node($parents[$item['id']]);
                    ksort($item['subcategories']);
                }
                $node[$item['id']] = $item;
            }
            return $node;
        };
        return $create_node(reset($parents));
    }

    public function list($limit, $limitstart) {
        return $this->getSimpleList('#__jshopping_categories', '*', 'category_id', 'asc', $limit, $limitstart);
    }

    public function listCount(): int {
        return $this->getSimpleListCount('#__jshopping_categories');
    }

}
