<?php
/*
* @version      2.0.4 06.09.2023
* @author       MAXXmarketing GmbH
* @package      addon_api
* @copyright    Copyright (C) 2010 webdesigner-profi.de. All rights reserved.
* @license      GNU/GPL
*/
namespace Joomla\Component\Jshopping\Site\Model\Api;
defined('_JEXEC') or die;

class OrderModel extends ApiModel {

    public function ids(): array {
        $db = \JFactory::getDbo();
        $db->setQuery('
            SELECT ' . $db->qn('order_id') . '
            FROM '   . $db->qn('#__jshopping_orders')
        );
        $res = array_map('intval', (array) $db->loadColumn());
        sort($res);
        return $res;
    }

    /**
     * @throws AddonApiException
     */
    public function item(string $id) {
        $addon = \AddonApi::getInst();
        $order = \JSFactory::getTable('order');
        $order->load($id);
        if (!$order->order_id) {
            throw new \AddonApiException_order(1, ' \'' . $id . '\'');
        }
        $order->prepareBirthdayFormat();
        $order->client_type_name = $order->getClientTypeName();
        $order->coupon_code      = $order->getCouponCode();
        $order->order_tax_list   = $order->getTaxExt();
        $order->payment_name     = $order->getPaymentName();
        \JSHelper::filterHTMLSafe($order);
        if ($addon->addonExists('addon_servicebox')) {
            $order->delivery_id = \AddonServicebox::getInst()->getModel('order')->getDeliveryId($order);
        }
        return $order;
    }

    public function states(): array {
        $res = [];
        foreach ((array) \JSFactory::getModel('orders')->getAllOrderStatus() as $state) {
            $res[$state->status_id] = [
                'id'   => (int)    $state->status_id,
                'code' => (string) $state->status_code,
                'name' => (string) $state->name
            ];
        }
        ksort($res);
        return $res;
    }

    public function list($limit, $limitstart) {
        return $this->getSimpleList('#__jshopping_orders', '*', 'order_id', 'asc', $limit, $limitstart);
    }

    public function listCount(): int {
        return $this->getSimpleListCount('#__jshopping_orders');
    }

    public function products(int $order_id) {
        $db = \JFactory::getDbo();        
        $query = $db->getQuery(true);
        $query->select('*');        
        $query->from($db->qn('#__jshopping_order_item'));
        $query->where($db->qn('order_id') . '=' . $db->q($order_id));
        $query->order($db->qn('order_item_id')." ASC");
        $db->setQuery($query);
        return $db->loadObjectList();        
    }

}
