<?php
/*
* @version      2.0.5 25.03.2024
* @author       MAXXmarketing GmbH
* @package      addon_api
* @copyright    Copyright (C) 2010 webdesigner-profi.de. All rights reserved.
* @license      GNU/GPL
*/
namespace Joomla\Component\Jshopping\Site\Model\Api;
use Joomla\Component\Jshopping\Site\Helper\Selects;
use Joomla\Component\Jshopping\Site\Helper\SelectOptions;    
defined('_JEXEC') or die;

class ProductModel extends ApiModel {

    public function getProductList(
        string $search          = '',
        string $search_type     = '',
        array  $categorys       = [],
        int    $include_subcat  = 1,
        array  $manufacturers   = [],
        array  $vendors         = [],
        array  $labels          = [],
        float  $price_from      = 0,
        float  $price_to        = 0,
        string $date_from       = '',
        string $date_to         = '',
        array  $extra_fields    = [],
        int    $order           = 0,
        int    $orderby         = 0,
        int    $limit           = 0,
        int    $limitstart      = 0
    ){
        $args    = get_defined_vars();
        
        $productlist = \JSFactory::getModel('Productsearch', 'Site\\Api');
        if ($order) {
            $productlist->setOrder($order);
        }
        if ($orderby) {
            $productlist->setOrderBy($orderby);
        }
        if ($limit) {
            $productlist->setLimit($limit);
        }
        if ($limitstart) {
            $productlist->setLimitStart($limitstart);
        }
        
		$filters = [];
        $filters['categorys'] = $categorys;
        $filters['manufacturers']= $manufacturers;
        $filters['price_from'] = $price_from;
        $filters['price_to'] = $price_to;        
        $filters['extra_fields'] = $extra_fields;        
        $filters['labels'] = $labels;
		$filters['search'] = $search;
		$filters['date_from'] = $date_from;
		$filters['date_to'] = $date_to;
		$filters['search_type'] = $search_type;
        //print_r($filters); die();

        $productlist->setFilters($filters);
        $productlist->load();
        return $productlist;
    }

    /**
     * @throws AddonApiException
     */
    public function group(string $group, int $extra_data = 0): array {
        $productlist = \JSFactory::getModel($group, 'Site\\Productlist');
        if ($productlist === false) {
            throw new \AddonApiException_product(2, ' \'' . $group . '\'');
        }
        $productlist->setMultiPageList(0);
        $productlist->load();
        $productlist->configDisableSortAndFilters();
        
        $view = $this->getView('products');
        if ($extra_data) {
            $res  = [
                'allow_review'          => (bool)  $productlist->getAllowReview(),
                'count_product_to_row'  => (int)   $productlist->getCountProductsToRow(),
                'display_list_products' => (bool)  $productlist->getDisplayListProducts(),
                'display_pagination'    => false,
                'products'              => (array) $productlist->getProducts()
            ];
        } else {
            $res  = [              
                'products'              => (array) $productlist->getProducts()
            ];
        }
        $this->triggerView('onBeforeDisplayProductListView', $res, [&$view, &$productlist]);
        return $res;
    }

    public function ids(): array {
        $db = \JFactory::getDbo();
        $db->setQuery('
            SELECT ' . $db->qn('product_id') . '
            FROM '   . $db->qn('#__jshopping_products')
        );
        $res = array_map('intval', (array) $db->loadColumn());
        sort($res);
        return $res;
    }

    /**
     * @throws AddonApiException
     */
    public function item(int $id, array $attributes = [], int $extra_data = 0): array {
        $category_id     = 0;
        $user            = \JFactory::getUser();
        $jshopConfig     = \JSFactory::getConfig();
        $model           = \JSFactory::getModel('productShop', 'jshop');
        $modelreviewlist = \JSFactory::getModel('productReviewList', 'jshop');
        $back_value      = $model->getBackValue($id, $attributes);
        $dispatcher      = \JFactory::getApplication();
        $dispatcher->triggerEvent('onBeforeLoadProduct', [&$id, &$category_id, &$back_value]);
        $dispatcher->triggerEvent('onBeforeLoadProductList', []);
        $product         = \JSFactory::getTable('product');
        $product->load($id);
        if (!$product->product_id) {
            throw new \AddonApiException_product(1, ' ' . $id);
        }
        $model->setProduct($product);
        $model->prepareView($back_value);
        $model->clearBackValue();
        $listcategory = $model->getCategories(1);
        $category_id  = reset($listcategory);
        $category     = \JSFactory::getTable('category');
        $category->load($category_id);
        $modelreviewlist->setModel($product);
        $modelreviewlist->load();
        $product->hit();
        $product_images    = $product->getImages();
        $product_videos    = $product->getVideos();
        $product_demofiles = $product->getDemoFiles();
        $view              = $this->getView('product');
        $dispatcher->triggerEvent('onBeforeDisplayProduct', [&$product, &$view, &$product_images, &$product_videos, &$product_demofiles]);
        if ($extra_data) {
            $res = [
                'all_attr_values'          => (array)  $model->getAllAttrValues(),
                'allow_review'             => (bool)   $model->getAllowReview(),
                'attributes'               => (array)  $model->getAttributes(),
                'available'                => (string) $model->getTextAvailable(),
                'back_value'               => (array)  $back_value,
                'default_count_product'    => (int)    $model->getDefaultCountProduct($back_value),
                'demofiles'                => (array)  $product_demofiles,
                'display_pagination'       => (bool)   $modelreviewlist->getPagination()->getPagesLinks(),
                'enable_wishlist'          => (bool)   $jshopConfig->enable_wishlist,
                'hide_buy'                 => (bool)   $model->getHideBuy(),
                'image_path'               => (string) $jshopConfig->live_path . 'images',
                'image_product_path'       => (string) $jshopConfig->image_product_live_path,
                'images'                   => (array)  $product_images,
                'noimage'                  => (string) $jshopConfig->noimage,
                'parts_count'              => (int)    $jshopConfig->rating_starparts,
                'product'                  =>          $product,
                'related_prod'             => (array)  $product->product_related,
                'reviews'                  => (array)  $modelreviewlist->getList(),
                'select_review'            => (array)  range(0, $jshopConfig->max_mark),
                'stars_count'              => (int)    floor($jshopConfig->max_mark / $jshopConfig->rating_starparts),
                'text_review'              => (string) $model->getTextReview(),
                'video_image_preview_path' => (string) $jshopConfig->video_product_live_path,
                'video_product_path'       => (string) $jshopConfig->video_product_live_path,
                'videos'                   => (array)  $product_videos
            ];
        } else {
            $res = [
                'product' => $product,
            ];
        }
        $this->triggerView('onBeforeDisplayProductView', $res, [&$view]);
        $dispatcher->triggerEvent('onAfterDisplayProduct', [&$product]);
        return $res;
    }

    public function search(
        string $search          = '',
        string $search_type     = '',
        array  $categories      = [],
        int    $include_subcat  = 1,
        array  $manufacturers   = [],
        array  $vendors         = [],
        array  $labels          = [],
        float  $price_from      = 0,
        float  $price_to        = 0,
        string $date_from       = '',
        string $date_to         = '',
        array  $extra_fields    = [],
        int    $order           = 0,
        int    $orderby         = 0,
        int    $limit           = 0,
        int    $limitstart      = 0
    ): array {
        $productlist = $this->getProductList(...func_get_args());
        $view        = $this->getView('search');
        /*$res         = [
            'allow_review'             => (bool)   $productlist->getAllowReview(),
            'count_product_to_row'     => (int)    $productlist->getCountProductsToRow(),
            'display_pagination'       => (bool)   $productlist->getPagenav() != '',
            'filters'                  => (array)  $productlist->getFilters(),
            'filter_show'              => false,
            'filter_show_category'     => false,
            'filter_show_manufacturer' => false,
            'orderby'                  => (string) $productlist->getOrderBy(),
            'pagination'               =>          $productlist->getPagination(),
            'path_image_sorting_dir'   => (string) \JSFactory::getConfig()->live_path . 'images/' . $productlist->getImageSortDir(),
            'product_count'            => (array)  SelectOptions::getProductsCount(),
            'products'                 => (array)  $productlist->getProducts(),
            'sorting'                  => (array)  SelectOptions::getProductsOrdering(),
            'total'                    => (int)    $productlist->getTotal()
        ];*/
        $res = [            
            'products'                 => (array)  $productlist->getProducts(),            
            'total'                    => (int)    $productlist->getTotal()
        ];
        $this->triggerView('onBeforeDisplayProductListView', $res, [&$view, &$productlist]);
        return $res;
    }

    public function searchInfo() {
        return [
            'categories'    => (array) SelectOptions::getCategories(0),
            'product_count' => (array) SelectOptions::getProductsCount(),
            'extra_fields'  => array_intersect_key(
                (array) \JSFactory::getAllProductExtraField(),
                array_flip((array) \JSFactory::getDisplayFilterExtraFieldForCategory(0))
            ),
            'labels'        => (array) SelectOptions::getLabels(0),
            'manufacturers' => (array) SelectOptions::getManufacturers(0),
            'order'         => (array) SelectOptions::getProductsOrdering(),
            'vendors'       => (array) SelectOptions::getVendors(0)
        ];
    }

    public function list($limit, $limitstart) {
        return $this->getSimpleList('#__jshopping_products', '*', 'product_id', 'asc', $limit, $limitstart);
    }

    public function listCount(): int {
        return $this->getSimpleListCount('#__jshopping_products');
    }

}
